/**
 * This is where all of the constants I'd like to use throughout the 
 *   ray-tracer will live
 * 
 * @author	Konstantin Shkurko (kshkurko@cs.utah.edu)
 * @date	09.11.11
 */
#ifndef CONSTANTS_H
#define CONSTANTS_H


#include "trax.hpp"			// for min/max definitions


/// raytracer constants namespace
namespace MYRTC {

	static const float GLOBAL_EPS = 1.e-7f;					/// global epsilon
	static const float GLOBAL_EPS_SQ = GLOBAL_EPS*GLOBAL_EPS;

	/**
	 * Some Camera FOV constants so we don't have to re-compute 
	 *   them all the time
	 */
	static const float CAM_FOV_90_DEG = 1.0f;				/// 90 deg fov
	static const float CAM_FOV_60_DEG = 0.577350269f;		/// 60 deg fov
	static const float CAM_FOV_45_DEG = 0.414213562f;		/// 45 deg fov
	static const float CAM_FOV_22_DEG = 0.194380309f;		/// 22 deg fov
	static const float CAM_FOV_193_DEG= 0.35f;				/// 19.3 deg fov


	/**
	 * Ray intersection point constants
	 */
	static const float RAY_EPS = 1.e-4f;					/// closest hit point for a ray
	static const float RAY_INF = 10000000.f;				/// furthest hit point for a ray
	static const float RAY_INTER_STEP = 1.e-4f;				/// step for a ray after a hit point


	/**
	 * Material properties that we can use for reflections/refractions
	 */
	static const float MATETA_AIR				= 1.000293f;
	static const float MATETA_ICE				= 1.31f;
	static const float MATETA_WATER				= 1.333333f;
	static const float MATETA_ETHYL_ALC			= 1.36f;
	static const float MATETA_FLOURITE			= 1.43f;
	static const float MATETA_POPPY_SEED_OIL	= 1.469f;
	static const float MATETA_OLIVE_OIL			= 1.47f;
	static const float MATETA_LINSEED_OIL		= 1.478f;
	static const float MATETA_ACRYLIC			= 1.49f;
	static const float MATETA_PLEXIGLASS		= 1.51f;
	static const float MATETA_IMMERSION_OIL		= 1.515f;
	static const float MATETA_CROWN_GLASS		= 1.52f;
	static const float MATETA_QUARTZ			= 1.54f;
	static const float MATETA_SALT				= 1.54f;
	static const float MATETA_LIGHT_FLINT_GLASS	= 1.58f;
	static const float MATETA_DENSE_FLINT_GLASS = 1.66f;
	static const float MATETA_TOURMALINE		= 1.62f;
	static const float MATETA_GARNET			= 1.73f;	// range: 1.73-1.89
	static const float MATETA_ZIRCON			= 1.923f;
	static const float MATETA_CUBIC_ZIRCONIA	= 2.14f;	// range: 2.14-2.20
	static const float MATETA_DIAMOND			= 2.417f;
	static const float MATETA_RUTILE			= 2.907f;
	static const float MATETA_GALLIUM_PHOSPHIDE	= 3.5f;


	/**
	 * Types of objects we have implemented -> this is intended
	 *   to be used by our inheritance-less code... and casting
	 *   void pointers :/
	 */
	typedef enum {
		OT_UNKNOWN = 0,			/// unknown object type
		OT_SPHERE,				/// sphere object type
		OT_TRIANGLE,			/// triangle object type
		OT_TRIANGLE_TRAX,		/// triangle object type, BUT must be loaded from TRAX!
		OT_BOX,					/// box object type
	} OBJ_TYPE;


	/**
	 * Clamping function to [0,1] range
	 */
	static inline const float clamp01( const float &v ) {
		return max( min( v, 1.f ), 0.f );
	}

	/**
	 * Clamping function to [A,B] range
	 */
	static inline const float clampAB( const float &v, const float &A, const float &B ) {
		return max( min( v, B ), A );
	}

	/**
	 * Swaps two values by messing with addresses
	 * @param a - a value
	 * @param b - b value
	 */
	static inline const void swap( float *aPtr, float *bPtr ) {
		float * const tmpPtr = aPtr;
		aPtr = bPtr;
		bPtr = tmpPtr;
	}
};

#endif // CONSTANTS_H
