/**
 * Basic abstract class for a ray primitive to be extended into shadow
 *   and primary rays
 *
 * A ray is basically defined by origin, direction, minT, and maxT
 *   So a ray is basically: r = O + D * t, with t in [minT, maxT]
 *	 This must be true: minT < maxT, even if maxT < 0
 *
 * @author: Konstantin Shkurko (kshkurko@cs.utah.edu)
 * @date:	09.04.11
 */
#ifndef RAY_PRIMITIVE_H
#define RAY_PRIMITIVE_H

#include "Constants.h"
#include "Vector.h"


/// abstract class definition
class RayPrimitive {

	public:
		/**
		 * Constructor for the ray
		 *
		 * @param org  - origin of the ray
		 * @param dir  - direction of the ray
		 * @param minT - minimum t parameter, s.t t<minT is outside ray
		 * @param maxT - maximum t parameter, s.t t>maxT is outside ray
		 */
		RayPrimitive( const Vector &org=Vector(0.f,0.f,0.f), const Vector &dir=Vector(1.f,0.f,0.f),
					const float &minT=MYRTC::RAY_EPS, const float &maxT=MYRTC::RAY_INF ) : 
					m_vOrigin(org), m_vDirection(dir) {

			// make sure minT and maxT are correct
			m_fMinT = minT;
			m_fMaxT = maxT;
	#ifdef DO_REFLECTIONS
			m_fCurEta = 1.f;
	#endif
		}


		/**
		 * Computes the info needed for box intersections...
		 * Using the values before calling this function is undefined!
		 */
		inline void computeBoxIntersectInfo() {
			m_vDirInv.set( 1.f/m_vDirection.x, 1.f/m_vDirection.y, 1.f/m_vDirection.z );
			m_iDirSignX = (m_vDirection.x < 0.f);
			m_iDirSignY = (m_vDirection.y < 0.f);
			m_iDirSignZ = (m_vDirection.z < 0.f);
		}


		/**
		 * Gets the location somewhere along a ray, based on 
		 *   the input t parameter (no check for in-boundedness!!!)
		 *
		 * @param ptOut - output point location
		 * @param tLoc  - t parameter to look up
		 * @return location
		 */
		inline void getPoint( Vector &ptOut, const float &tLoc ) const {
			ptOut.set( m_vDirection );
			ptOut.mult( tLoc );
			ptOut.add( m_vOrigin );
		}


		// public for speed
		Vector m_vOrigin;			/// origin of a ray
		float m_fMaxT;				/// maximum t for intersections
		Vector m_vDirection;		/// normalized direction of ray
		float m_fMinT;				/// minimum t for intersections
		Vector m_vDirInv;			/// inverse direction vector (aka 1./dx...)
		unsigned int m_iDirSignX;	/// direction of sign in X ( aka +(1) or -(0) ) done via bit-setting
		unsigned int m_iDirSignY;	/// direction of sign in Y ( aka +(1) or -(0) ) done via bit-setting
		unsigned int m_iDirSignZ;	/// direction of sign in Z ( aka +(1) or -(0) ) done via bit-setting
	#ifdef DO_REFLECTIONS
		float m_fCurEta;			/// current eta for dieletric materials
	#endif
};


#endif // RAY_PRIMITIVE_H
